<?php
if (!defined("CLUEVO_ACTIVE")) exit;

require_once(plugin_dir_path(__DIR__) . "conf/config.inc.php");
require_once(plugin_dir_path(__DIR__) . "functions/functions.inc.php");

function cluevo_api_get_modules() {
  $modules = cluevo_get_modules();
  return $modules;
}

function cluevo_api_get_module($args) {
  $module = cluevo_get_module($args["module_id"]);
  return $module;
}

function cluevo_api_upload_module(WP_REST_Request $request) {
  $errors = [];
  $messages = [];
  $handled = false;
  $module = null;

  if (!empty($_FILES) && !empty($_FILES["module-file"]) && $_FILES["module-file"]["error"] === UPLOAD_ERR_OK) {
    if (!empty($_FILES["module-file"])) {
      cluevo_handle_module_upload($errors, $messages, $handled, $module);
    }
  } else {
    if (!empty($request["module-dl-url"])) {
      if (filter_var($request["module-dl-url"], FILTER_VALIDATE_URL)) {
        $url = esc_url_raw($request["module-dl-url"], ['http', 'https', 'ftp']);
        if (!empty($url)) {
          if (cluevo_url_exists($url) || $request["force"] === "true") {
            cluevo_handle_module_download($request["module-dl-url"], $errors, $messages, $handled, $module);
          } else {
            $errors[] = __("Url does not exist", "cluevo");
          }
        }
      } else {
        do_action('cluevo_handle_misc_module_url_input', [
          "input" => $request["module-dl-url"],
          "handled" => &$handled,
          "result" => &$result,
          "errors" => &$errors,
          "messages" => &$messages,
          "result" => &$module
        ]);
        if (!$handled) {
          $messages[] = __("No handler for this module type could be found.", "cluevo");
        }
      }
    }
  }

  if (!$handled) {
    return new WP_REST_Response([ 'errors' => $errors, 'messages' => $messages, 'handled' => $handled, "module" => $module ], 400);
  } else {
    return [ 'errors' => $errors, 'messages' => $messages, 'handled' => $handled, "module" => $module ];
  }
}

function cluevo_api_fetch_module_parameters($args) {
  $userId = get_current_user_id();
  if ($userId > 0) {
    $item = cluevo_get_learning_structure_item($args["item_id"], $userId);
    $module = cluevo_get_module($item->module_id);
    $parms = cluevo_get_module_parameters($item->module_id, $userId);
    $parms["_scorm_version"] = $module->scorm_version;
    return $parms;
  } else {
    $item = cluevo_get_learning_structure_item($args["item_id"]);
    $module = cluevo_get_module($item->module_id);
    $parms = cluevo_get_module_parameters($item->module_id);
    $parms["_scorm_version"] = $module->scorm_version;
    return $parms;
  }
}

function cluevo_api_init_new_attempt(WP_REST_Request $request) {
  $userId = get_current_user_id();
  if ($userId > 0) {
    $item = cluevo_get_learning_structure_item($request["item_id"], $userId);
    $intModuleId = $item->module_id;
    if (empty($intModuleId)) {
      return "invalid module";
    }
    $moduleData = cluevo_get_module($intModuleId);
    $version = $moduleData->scorm_version;
    $state = cluevo_init_module_progress($userId, $intModuleId, [], $version);
    $parms = cluevo_get_module_parameters($intModuleId, $userId, $state["attempt_id"]);
    return $parms;
  } else {
    return "authentication required";
  }
}

function cluevo_api_fetch_learning_structure_item($args) {
  $userId = get_current_user_id();
  return cluevo_get_learning_structure_item($args["item_id"], $userId);
}

function cluevo_api_fetch_learning_structure_items() {
  $userId = get_current_user_id();
  $items = cluevo_get_learning_structure_items(0, $userId);

  return $items;
}

function cluevo_save_module_progress(WP_REST_Request $request) {
  $userId = get_current_user_id();
  if ($userId > 0) {
    $module = cluevo_get_module((int)$request['id']);
    if (!empty($module)) {
      switch ($module->type_name) {
        case "audio":
        case "video":
          cluevo_update_media_module_progress($userId, (int)$request['id'], (float)$request['max'], (float)$request["score"]);
          break;
        default:
          do_action('cluevo_save_module_progress', [ "user_id" => $userId, "module" => $module, "request" => $request ]);
      }
    }
  } else {
    return "authentication required";
  }
  return true;
}

function cluevo_save_module_data(WP_REST_Request $request) {
  $userId = get_current_user_id();
  if ($userId > 0) {
    return cluevo_write_module_parameters($request['id'], $userId, $request->get_json_params());
  } else {
    return "authentication required";
  }
}

function cluevo_api_create_item(WP_REST_Request $request) {
  $userId = get_current_user_id();
  $parentId = (int)$request['id'];
  $parentPostId = 0;
  $level = 0;
  if ($parentId != 0) {
    $parent = cluevo_get_learning_structure_item($parentId, $userId);
    if (!empty($parent)) {
      $parentPostId = $parent->metadata_id;
      $level = $parent->level + 1;
    } else {
      return new WP_Error( 'error', 'parent item not found' );
    }
  }
  $tmpItem = new CluevoItem();
  $tmpItem->parent_id = $parentId;
  $tmpItem->level = $level;
  switch ($level) {
    case 1:
      $name = __("New Course", "cluevo");
      break;
    case 2:
      $name = __("New Chapter", "cluevo");
      break;
    case 3:
      $name = __("New Module", "cluevo");
      break;
    default:
      $name = __("New Item", "cluevo");
      
  }
  $tmpItem->name = $name;
  $metaId = cluevo_create_metadata_page($tmpItem, $parentPostId);
  $tmpItem->metadata_id = $metaId;
      
  return cluevo_create_learning_structure_item($tmpItem);
}

function cluevo_api_make_item_module(WP_REST_Request $request) {
  $id = (int)$request["id"];
  $moduleId = (int)$request["mid"];
  $userId = get_current_user_id();
  $item = cluevo_get_learning_structure_item($id, $userId);
  $module = cluevo_get_module($moduleId);
  if (!empty($item) && !empty($module)) {
    //cluevo_create_learning_structure_module_item($id, $moduleId);
    $item->module_id = $moduleId;
    cluevo_update_learning_structure_item($item);
  }
}

function cluevo_api_remove_item_module(WP_REST_Request $request) {
  $id = (int)$request["id"];
  $userId = get_current_user_id();
  $item = cluevo_get_learning_structure_item($id, $userId);
  $module = cluevo_get_module($moduleId);
  if (!empty($item)) {
    cluevo_remove_learning_structure_module_item($id);
  }
}

function cluevo_api_delete_item(WP_REST_Request $request) {
  $id = (int)$request['id'];
  return cluevo_remove_learning_structure_item($id);
}

function cluevo_api_change_module_language(WP_REST_Request $request) {
  $id = (int)$request['id'];
  $from = $request['from'];
  $to = $request['to'];
  //return [$id, $langCode ];

  return cluevo_set_module_language($id, $from, $to);
}

function cluevo_api_update_module_name(WP_REST_Request $request) {
  $id = (int)$request['id'];
  $name = $request['name'];
  $module = cluevo_get_module($id);
  $result = false;
  if ($module) {
    if (!cluevo_module_exists($name)) {
      $result = cluevo_update_module_name($id, $name);
    }
  }
  return $result;
}

add_action( 'rest_api_init', function () {

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_get_modules'
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/upload', array(
    'methods' => 'POST',
    'callback' => 'cluevo_api_upload_module',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<module_id>[\d]+)', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_get_module'
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<item_id>[\d]+)/parameters', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_fetch_module_parameters',
    'args' => [ 'item_id' ]
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<item_id>[\d]+)/new-attempt', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_init_new_attempt',
    'args' => [ 'item_id' ]
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/items/(?P<item_id>[\d]+)', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_fetch_learning_structure_item',
    'args' => [ 'item_id' ]
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/items', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_fetch_learning_structure_items',
    //'permission_callback' => function () {
      //return current_user_can( 'administrator' );
    //}
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/items/(?P<id>[\d]+)/create', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_create_item',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/items/(?P<id>[\d]+)/make-module/(?P<mid>[\d]+)', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_make_item_module',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/items/(?P<id>[\d]+)/remove-module', array(
    'methods' => 'GET',
    'callback' => 'cluevo_api_remove_item_module',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/items/(?P<id>[\d]+)/delete', array(
    'methods' => 'DELETE',
    'callback' => 'cluevo_api_delete_item',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<id>[\d-]+)/parameters', array(
    'methods' => 'PUT',
    'callback' => 'cluevo_save_module_data'
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<id>[\d-]+)/progress', array(
    'methods' => 'POST',
    'callback' => 'cluevo_save_module_progress'
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<id>[\d]+)/language', array(
    'methods' => 'POST',
    'callback' => 'cluevo_api_change_module_language',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );

  register_rest_route( CLUEVO_PLUGIN_NAMESPACE . '/v1', '/modules/(?P<id>[\d-]+)/name', array(
    'methods' => 'POST',
    'callback' => 'cluevo_api_update_module_name',
    'permission_callback' => function () {
      return current_user_can( 'administrator' );
    }
  ) );
} );

include "routes.competences.php";
include "routes.user-management.php";
include "routes.permissions.php";
include "routes.user.php";
?>
